<?php
/**
 * The plugin bootstrap file
 *
 *
 * @link              https://www.centralpay.com
 * @since             2.2.0
 * @package           cpay
 *
 * @wordpress-plugin
 * Plugin Name:       CentralPay
 * Plugin URI:        http://wordpress.org/plugins/cpay/
 * Description:       Réalisez des paiements par carte de crédit avec CentralPay.
 * Version:           2.2.0
 * Author:            CentralPay
 * Author URI:        https://www.centralpay.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       cpay
 * Domain Path:       /languages
 * Slug:              cpay
 */

if (!defined('WPINC')) {
    die;
}

/**
 * Currently plugin version.
 */
define('CPAY_VERSION', '2.2.0');

if (!function_exists('is_plugin_active')) {
    include_once(ABSPATH . '/wp-admin/includes/plugin.php');
}

if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

    function activateCpay()
    {
        require_once plugin_dir_path(__FILE__) . 'includes/class-cpay-activator.php';
        CpayActivator::activate();
    }


    function deactivateCpay()
    {
        require_once plugin_dir_path(__FILE__) . 'includes/class-cpay-deactivator.php';
        CpayDeactivator::deactivate();
    }

    function cpay_missing_wc_notice()
    {
        $class = 'notice notice-error';
        $message = __('The centralpay plugin requires WooCommerce to be installed and active.', 'cpay');

        printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
    }

    function cpay_check_requirements()
    {
        if (is_plugin_active('woocommerce/woocommerce.php')) {
            return true;
        } else {
            add_action('admin_notices', 'cpay_missing_wc_notice');
            return false;
        }
    }

    add_action('plugins_loaded', 'cpay_check_requirements');

    register_activation_hook(__FILE__, 'activateCpay');
    register_deactivation_hook(__FILE__, 'deactivateCpay');

    require plugin_dir_path(__FILE__) . 'includes/class-cpay.php';

    add_action('before_woocommerce_init', 'cpay_declare_cart_checkout_blocks_compatibility');

    function cpay_declare_cart_checkout_blocks_compatibility()
    {
        if (class_exists('\Automattic\WooCommerce\Utilities\FeaturesUtil')) {
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('cart_checkout_blocks', __FILE__, true);
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
        }
    }

    add_action('woocommerce_blocks_loaded', 'cpay_woocommerce_blocks_support');

    function cpay_woocommerce_blocks_support()
    {
        if (class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
            require_once plugin_dir_path(__FILE__) . 'includes/class-cpay-wc-gateway_blocks.php';
            add_action(
                'woocommerce_blocks_payment_method_type_registration',
                /** @phpstan-ignore-next-line */
                function (Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry) {
                    /** @phpstan-ignore-next-line */
                    $payment_method_registry->register(new Cpay_wc_Gateway_Blocks());
                }
            );


            add_filter('woocommerce_blocks_checkout_blocks', function ($blocks) {
                $blocks['cpay/checkout'] = array(
                    'title' => __('CentralPay', 'yzohou'),
                );
                return $blocks;
            });


        }
    }

    function run_cpay()
    {
        if (cpay_check_requirements()) {
            $plugin = new Cpay();
            $plugin->run();
        }
    }

    run_cpay();


}
