<?php

if (!defined("ABSPATH")) {
    exit;
}

require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cpay-log.php';

class CpayApi
{
    /**
     * Send a POST request using cURL
     * @param string $url to request
     * @param array $post values to send
     * @return array column in the array ['header','body']
     */
    public static function curl_post($url, $userpwd, array $post = null)
    {
        if (empty($url) or empty($userpwd)) {
            throw new Exception("An error was occured when the CURl POST has called, a parameters was empty.");
        } else {
            try {
                $defaults = array(
                    CURLOPT_URL => $url,
                    CURLOPT_RETURNTRANSFER => 1,
                    CURLOPT_POSTFIELDS => http_build_query($post),
                    CURLOPT_POST => 1,
                    CURLOPT_SSL_VERIFYPEER => false,
                    CURLOPT_HEADER => 1,
                    CURLOPT_USERPWD => $userpwd
                );

                $curl = curl_init();
                curl_setopt_array($curl, $defaults);

                $result = curl_exec($curl);

                $info = curl_getinfo($curl);
                $header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);

                curl_close($curl);

                $body = substr($result, $header_size);

                return array(
                'http_code' => $info['http_code'],
                'body' => $body
                );
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }

    /**
     * Send a GET requst using cURL
     * @param string $url to request
     * @return array
     * @throws Exception
     */
    public static function curl_get_api($url, $userpwd, array $post = null, $type = 'POST')
    {
        if (empty($url) or empty($userpwd)) {
            /** @phpstan-ignore-next-line */
            wc_add_notice(__('An error occurred during the processing of your order', 'cpay'), 'error');
            /** @phpstan-ignore-next-line */
            wc_get_logger()->error('Empty URL or Centralpay API user password.');
            $satuslogin = (empty($userpwd) ? null : "********");
            CpayLog::newLog("[ERROR]class-cpay-api.php l.".__LINE__." EMPTY VALUE, {{asked(string] url : sent(".gettype($url).") ".$url."},{asked userpwd : sent(".gettype($userpwd).") ".$satuslogin."}}");

            throw new Exception("An error was occured when the CURl POST has called, a parameters was empty.");
        } else {
            try {
                $curl = curl_init();

                curl_setopt_array($curl, array(
                    CURLOPT_URL => $url,
                    CURLOPT_USERPWD => $userpwd,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => '',
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => $type,
                    CURLOPT_POSTFIELDS => $post,
                    CURLOPT_HTTPHEADER => array(
                        'Origin: '.get_site_url(),
                    ),
                ));

                $response = curl_exec($curl);
                $info = curl_getinfo($curl);
                $header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);

                curl_close($curl);

                if (empty($info['http_code'])) {
                    /** @phpstan-ignore-next-line */
                    wc_add_notice(__('No http code', 'cpay'), 'error');
                    /** @phpstan-ignore-next-line */
                    wc_get_logger()->error("[ERROR]class-cpay-api.php l.".__LINE__." CURL GET API No HTTP code was returned");

                    CpayLog::newLog("[ERROR]class-cpay-api.php l.".__LINE__." CURL GET API No HTTP code was returned");
                    throw new Exception("No HTTP code was returned");
                }

                $body = substr($response, $header_size);
                /** @phpstan-ignore-next-line */
                wc_get_logger()->info("Code is: ". $info['http_code'] . " and Response is: ". $response);
                return array(
                    'http_code' => $info['http_code'],
                    'reponse' => json_decode($response),
                    'body' => $body
                );
            } catch (Exception $e) {
                /** @phpstan-ignore-next-line */
                wc_get_logger()->error("[ERROR]class-cpay-api.php l.".__LINE__." ". $e->getMessage());
                CpayLog::newLog("[ERROR]class-cpay-api.php l.".__LINE__." ". $e->getMessage());
                throw new Exception($e->getMessage());
            }
        }
    }
}
