<?php

require_once plugin_dir_path(__FILE__) . 'class-cpay-config.php';
require_once plugin_dir_path(__FILE__) . 'class-cpay-api.php';
require_once plugin_dir_path(__FILE__) . 'class-cpay-log.php';

class WC_CPay_Gateway extends WC_Payment_Gateway
{
    private $login_cpay;
    private $password_cpay;
    private $merchant_public_key;
    private $point_of_sale_id;
    private $oneclick;
    /**
     * @var string
     */
    private $urlApi;

    public function __construct()
    {
        /** @phpstan-ignore-next-line */
        $this->id = 'cpay'; // payment gateway plugin ID
        /** @phpstan-ignore-next-line */
        $this->icon = plugin_dir_url(__FILE__) . 'images/logo-centralpay.png'; // URL of the icon that will be displayed on checkout page near your gateway name
        /** @phpstan-ignore-next-line */
        $this->has_fields = false; // in case you need a custom credit card form
        /** @phpstan-ignore-next-line */
        $this->method_title = __('CentralPay', 'cpay');
        /** @phpstan-ignore-next-line */
        $this->method_description = __('CentralPay allow you to pay online.', 'cpay');
        /** @phpstan-ignore-next-line */
        $this->order_button_text = __('Proceed to payment', 'cpay');
        /** @phpstan-ignore-next-line */
        $this->title = __('Paid with card from CentralPay', 'cpay');
        /** @phpstan-ignore-next-line */
        $this->supports = array('products');

        $this->init_form_fields();
        /** @phpstan-ignore-next-line */
        $this->init_settings();
        /** @phpstan-ignore-next-line */
        if ($this->get_option('mode_test') === 'yes') {
            CpayConfig::$TESTMODE = true;
        }
        $this->urlApi = CpayConfig::getApiUrl();
        /** @phpstan-ignore-next-line */
        $this->login_cpay = $this->get_option('login_cpay');
        /** @phpstan-ignore-next-line */
        $this->password_cpay = $this->get_option('password_cpay');
        /** @phpstan-ignore-next-line */
        $this->merchant_public_key = $this->get_option('merchant_public_key');
        /** @phpstan-ignore-next-line */
        $this->point_of_sale_id = $this->get_option('point_of_sale_id');
        /** @phpstan-ignore-next-line */
        $this->oneclick = $this->get_option('one_click');
        /** @phpstan-ignore-next-line */
        if (empty($this->get_option("salt"))) {
            /** @phpstan-ignore-next-line */
            $this->update_option('salt', CpayConfig::generateSalt());
        }

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_api_cpay_paid', array($this, 'webhook'));
        add_action('woocommerce_api_cpay_validation', array($this, 'pr_validation'));
    }

    public function init_form_fields()
    {
        /** @phpstan-ignore-next-line */
        $this->form_fields = apply_filters('wc_cpay_settings', array(
            'enabled' => array(
                'title' => __('Activate / Deactivate', 'woocommerce-gateway-cpay'),
                'label' => __('Activate cpay', 'woocommerce-gateway-cpay'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no'
            ),
            'login_cpay' => array(
                'title' => __('Login', 'woocommerce-gateway-cpay'),
                'type' => 'text',
                'description' => __('Login d\'accès à CPAY V2', 'woocommerce-gateway-cpay'),
                'default' => __('', 'woocommerce-gateway-cpay')
            ),
            'password_cpay' => array(
                'title' => __('Password', 'woocommerce-gateway-cpay'),
                'type' => 'password',
                'description' => __('Mot de passe d\'accès à CPAY V2', 'woocommerce-gateway-cpay'),
                'default' => __('', 'woocommerce-gateway-cpay')
            ),
            'merchant_public_key' => array(
                'title' => __('Merchant Public Key', 'woocommerce-gateway-cpay'),
                'type' => 'text',
                'description' => 'Clé publique pour la création des tokens.',
                'default' => ''
            ),
            'point_of_sale_id' => array(
                'title' => __('Point of sale id', 'woocommerce-gateway-cpay'),
                'type' => 'text',
                'description' => '',
                'default' => ''
            ),
            'one_click' => array(
                'title' => __('Activate OneClick', 'woocommerce-gateway-cpay'),
                'type' => 'checkbox',
                'label' => __('Activate OneClick', 'cpay'),
                'description' => '',
                'default' => 'no'
            ),
            'mode_test' => array(
                'title' => __('Modes: Production/Test', 'woocommerce-gateway-cpay'),
                'label' => __('Activer mode test', 'woocommerce-gateway-cpay'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no'
            ),

            /* Décommenter pour utiliser.
            * Il faut aussi ajouter le getter dans le constructeur de la classe class-wc-gateway-cpay.php
            */
            //        'mode_debug' => array(
            //            'title' => __( 'Modes: Debug', 'woocommerce-gateway-cpay' ),
            //            'label' => __( 'Activer mode debug', 'woocommerce-gateway-cpay' ),
            //            'type' => 'checkbox',
            //            'description' => '',
            //            'default' => 'no'
            //        ),

        ));
    }

    public function process_payment($order_id)
    {
        global $woocommerce;
        if (!empty($order_id) && is_int($order_id)) {
            $userpwd = $this->login_cpay . ":" . $this->password_cpay;
            /** @phpstan-ignore-next-line */
            $order = wc_get_order($order_id);
            $reference =  $this->merchant_public_key."-".$order_id;
            $amount = $order->get_total() * 100;

            $urlApiCheckPr = $this->urlApi . "paymentRequest?merchantPaymentRequestId=".$reference;

            $requeteCheckPr = CpayApi::curl_get_api($urlApiCheckPr, $userpwd, [], "GET");

            if ($requeteCheckPr['http_code'] == 200) {
                $reponseCheck = $requeteCheckPr['reponse'];
                if (!empty($reponseCheck->totalCount)) {
                    foreach ($reponseCheck->list as $paymentRequest) {
                        $amountPr = $paymentRequest->totalAmount;
                        $requete['reponse'] = $paymentRequest;
                        if ($amountPr == $amount) {
                            $urlredirect = Cpay::getUrlSmartForm($order_id, $requete);
                            if (!empty($urlredirect)) {
                                return array(
                                    'result' => 'success',
                                    'redirect' => $urlredirect,
                                );
                            }
                            CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." GET URL FORM EMPTY, {asked(int) order_id : sent(".gettype($order_id).") ".$order_id.", requete : ".json_encode($requete)."}");
                        }
                    }
                }
            } else {
                /** @phpstan-ignore-next-line */
                wc_get_logger()->error("[ERROR]class-cpay-api.php l.".__LINE__." EMPTY VALUE, received code is: ".$requeteCheckPr['http_code']);
                /** @phpstan-ignore-next-line */
                wc_add_notice(__('An error occurred during the processing of your order', 'cpay'), 'error');

            }


            $email = $order->get_billing_email();

            $breakdown = ["amount" => $amount,"email" => $email];
            $args = array(
                'paymentMethod[]' => 'TRANSACTION',
                'currency' => 'EUR',
                'totalAmount' => $amount,
                'pointOfSaleId' => $this->point_of_sale_id,
                'merchantPaymentRequestId' => $reference,
                'language' => substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2)
            );
            if ($this->oneclick === "yes") {
                /** @phpstan-ignore-next-line */
                $checkUser = DB::getCustomerId(get_current_user_id(), DB_NAME, DB_PASSWORD);
                if (!empty($checkUser)) {
                    $breakdown['customerId'] = $checkUser;
                } else {
                    $args['createCustomer'] = 'true';
                }
            }

            $args['breakdown[]'] = json_encode($breakdown);

            $urlApi = $this->urlApi . "paymentRequest";
            $requete = CpayApi::curl_get_api($urlApi, $userpwd, $args);

            if ($requete['http_code'] == 200) {
                $urlredirect = Cpay::getUrlSmartForm($order_id, $requete);
                if (!empty($urlredirect)) {
                    return array(
                        'result' => 'success',
                        'redirect' => $urlredirect,
                    );
                }
                /** @phpstan-ignore-next-line */
                wc_add_notice(__('An error occurred during the processing of your order', 'cpay'), 'error');
                CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." GET URL FORM EMPTY, {asked(int) order_id : sent(".gettype($order_id).") ".$order_id.", requete : ".json_encode($requete)."}");

                return;
            } else {
                /** @phpstan-ignore-next-line */
                wc_add_notice(__('Service connection error', 'cpay'), 'error');
                CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." CALL API, ['http_code' =>".$requete['http_code']."] {{asked(string) url : sent(".gettype($urlApi).") ".$urlApi."},{asked(array) arg : sent(".gettype($args).")".json_encode($args)."}}");

                return;
            }
        }
        /** @phpstan-ignore-next-line */
        wc_add_notice(__('An error occurred during the processing of your order', 'cpay'), 'error');
        CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." PROCESS_PAYMENT \"order_id require\", {asked(int) order_id : sent(".gettype($order_id).") " .$order_id.")}");

        return;
    }

    public function webhook()
    {
        if (!empty($_GET['idOrder'])) {
            $urlApi = $this->urlApi . "paymentRequest/".$_GET['paymentRequestId'];
            $userpwd = $this->login_cpay . ":" . $this->password_cpay;

            if ($this->oneclick === "yes") {
                $this->create_user($urlApi, $userpwd);
            }

            $orderId = Cpay::decrypteData($_GET['idOrder']);
            $order = $this->valid_payment($orderId);
            if (!empty($order)) {
                /** @phpstan-ignore-next-line */
                wp_redirect($this->get_return_url($order));
                exit();
            }

            CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." ID ORDER NOT MATCH, {sent GET['idOrder']=> ".$_GET['idOrder']."}");
            /** @phpstan-ignore-next-line */
            wc_add_notice(__("An error occurred while processing your request", "cpay"), 'notice');
            wp_redirect(get_permalink(get_option('woocommerce_myaccount_page_id')));
            exit();
        }
        /** @phpstan-ignore-next-line */
        wc_add_notice(__("An error occurred while processing your request", "cpay"), 'notice');
        CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." WEBHOOK ID ORDER IS EMPTY \"GET[idOrder] require\", {asked(int) idOrder : sent(".gettype($_GET['idOrder'])." ".$_GET['idOrder'].")}");
        // Redirection vers la page compte de l'utilisateur
        wp_redirect(get_permalink(get_option('woocommerce_myaccount_page_id')));
    }

    public function pr_validation()
    {
        if (!empty(json_decode(file_get_contents('php://input'))->object)) {
            if (!empty(json_decode(file_get_contents('php://input'))->object->transactionStatus) && json_decode(file_get_contents('php://input'))->object->transactionStatus === "SUCCESS") {
                $paymentRequest = json_decode(file_get_contents('php://input'))->object;
                $marchandTransactionId = $paymentRequest->merchantTransactionId;

                $arrayTransactionId = explode("-", $marchandTransactionId);
                $idMerchand = $arrayTransactionId[0];
                $orderId = end($arrayTransactionId);
                /** @phpstan-ignore-next-line */
                $order = wc_get_order($orderId);

                if (!empty($orderId && $order->status == "pending") && $idMerchand === $this->merchant_public_key) {
                    $urlApi = $this->urlApi . "paymentRequest/".$paymentRequest->paymentRequestId;

                    $userpwd = $this->login_cpay . ":" . $this->password_cpay;

                    if ($this->oneclick === "yes") {
                        $this->create_user($urlApi, $userpwd);
                    }

                    $order = $this->valid_payment($orderId);

                    if (!empty($order)) {
                        echo "order ". $orderId." status : ".$order->status;
                        exit();
                    }
                    CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." ID ORDER NOT MATCH, {sent GET['idOrder']=> ".$orderId."}");
                    /** @phpstan-ignore-next-line */
                    wc_add_notice(__("An error occurred while processing your request", "cpay"), 'notice');
                    // Redirection vers la page compte de l'utilisateur
                    wp_redirect(get_permalink(get_option('woocommerce_myaccount_page_id')));
                    exit();
                }
                /** @phpstan-ignore-next-line */
                wc_add_notice(__("An error occurred while processing your request", "cpay"), 'notice');
                CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." WEBHOOK transaction id IS INVALID : ".$arrayTransactionId);
                // Redirection vers la page compte de l'utilisateur
                wp_redirect(get_permalink(get_option('woocommerce_myaccount_page_id')));
                exit();
            }
        }
    }

    public function create_user($urlApi, $userpwd)
    {
        /** @phpstan-ignore-next-line */
        $checkUser = DB::getCustomerId(get_current_user_id(), DB_NAME, DB_PASSWORD);

        if (empty($checkUser)) {
            $requete = CpayApi::curl_get_api($urlApi, $userpwd, [], 'GET');

            if (!empty($requete['reponse']->breakdowns) && !empty($requete['reponse']->breakdowns[0]->customerId)) {
                $transactionData = [];
                $transactionData['customerId'] = $requete['reponse']->breakdowns[0]->customerId;
                $transactionData['userId'] = get_current_user_id();
                /** @phpstan-ignore-next-line */
                DB::createCustomer($transactionData, DB_USER, DB_PASSWORD);
            } else {
                CpayLog::newLog("[ERROR]cpay.php l.".__LINE__." GET CUSTOMERID TRANSACTION BAD VALUE BREAKDOWS/CUSTOMERID, {{asked(array) requete['reponse']->breakdown : sent(".gettype($requete['reponse']->breakdowns).")".json_encode($requete['reponse']->breakdowns)."},{asked(string) customerID : sent(".gettype($requete['reponse']->breakdowns[0]->customerId).") ".$requete['reponse']->breakdowns[0]->customerId."}");
            }
        }
    }

    public function valid_payment($orderId)
    {
        /** @phpstan-ignore-next-line */
        $order = wc_get_order($orderId);
        if (!empty($order)) {
            $order->payment_complete();
            //            $order->reduce_order_stock();
            /** @phpstan-ignore-next-line */
            wc_reduce_stock_levels($orderId);
            return $order;
        }
    }
}
