<?php

require_once plugin_dir_path(__FILE__) . 'class-cpay-wc-gateway.php';
use Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType;

final class Cpay_wc_Gateway_Blocks extends AbstractPaymentMethodType
{
    private $gateway;
    protected $name = 'cpay';

    public function initialize()
    {
        /** @phpstan-ignore-next-line */
        $this->settings = get_option('woocommerce_cpay_settings', []);
        $this->gateway = new WC_CPay_Gateway();
    }

    public function is_active()
    {
        return !empty($this->settings['enabled']) && 'yes' === $this->settings['enabled'];
    }


    public function get_payment_method_script_handles()
    {

        $script_url        =  plugins_url('/', __FILE__) . '/js/blocks.js';
        $script_asset_path =  plugins_url('/', __FILE__) . '/js/blocks.asset.php';
        $script_asset      = file_exists($script_asset_path)
            ? require($script_asset_path)
            : array(
                'dependencies' => array(),
                'version'      => '1.0.0'
            );

        wp_register_script(
            'wc-cpay-payments-blocks',
            $script_url,
            $script_asset[ 'dependencies' ],
            $script_asset[ 'version' ],
            true
        );

        return [ 'wc-cpay-payments-blocks' ];
    }



    /**
    * @return array
    */
    public function get_payment_method_data()
    {
        /** @phpstan-ignore-next-line */
        $title = $this->get_setting('title');
        /** @phpstan-ignore-next-line */
        $description = $this->get_setting('description');
        return [
            'title'       => $title,
            'description' => $description,
            'supports'    => array_filter($this->gateway->supports, [ $this->gateway, 'supports' ])
        ];
    }

}
