<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://www.centralpay.eu/
 * @since      1.0.0
 *
 * @package    cpay
 * @subpackage cpay/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    cpay
 * @subpackage cpay/includes
 * @author     CentralPay
 */

class Cpay
{
    public const ENCRIPTION_IV = "1234567891011121";

    protected $loader;
    protected $plugin_name;
    protected $version;

    public function __construct()
    {
        if (defined('CPAY_VERSION')) {
            $this->version = CPAY_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'cpay';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    public static function encrypteData($order)
    {
        if (!empty($order) && is_int($order)) {
            $ciphering = "AES-128-CTR";
            $iv_length = openssl_cipher_iv_length($ciphering);
            $options = 0;
            $salt = get_option('woocommerce_cpay_settings')['salt'];
            $encryption_key = "cpay" . $salt;
            $encryption_iv = Cpay::ENCRIPTION_IV;
            return openssl_encrypt(
                $order,
                $ciphering,
                $encryption_key,
                $options,
                $encryption_iv
            );
        }
        CpayLog::newLog("[ERROR]class-cpay.php l.".__LINE__." ENCRYPT DATA BAD VALUE, {asked(int) order : sent(".gettype($order).") ".$order."}");
        return "";
    }

    public static function decrypteData($encryption)
    {
        if (!empty($encryption)) {
            $ciphering = "AES-128-CTR";
            $iv_length = openssl_cipher_iv_length($ciphering);
            $options = 0;
            $salt = get_option('woocommerce_cpay_settings')['salt'];
            $encryption_key = "cpay" . $salt;
            $encryption_iv = Cpay::ENCRIPTION_IV;
            return openssl_decrypt(
                $encryption,
                $ciphering,
                $encryption_key,
                $options,
                $encryption_iv
            );
        }
        CpayLog::newLog("[ERROR]class-cpay.php l.".__LINE__." DECRYPT DATA EMPTY ENCRYPTION, {asked encryption : sent(".gettype($encryption).") ".$encryption."}");
        return "";
    }

    public static function getUrlSmartForm($order_id, $requete)
    {
        if ((!empty($requete) && !empty($requete['reponse'])) && (!empty($order_id) && is_int($order_id))) {
            if (!empty($requete['reponse']->breakdowns) && !empty($requete['reponse']->breakdowns[0]->endpoint)) {
                $response = $requete['reponse']->breakdowns[0]->endpoint;
                $paymentRequestId = $requete['reponse']->paymentRequestId ?? null;
                $orderIdEncrypte = Cpay::encrypteData($order_id);
                if (!empty($orderIdEncrypte) && $paymentRequestId !== null) {
                    //@phpstan-ignore-next-line
                    $urlredirect = $response.'?urlRedirect='.get_site_url().'?wc-api=cpay_paid%26idOrder='.$orderIdEncrypte.'%26paymentRequestId='.$paymentRequestId.'&delay=3&urlOnCanceled='.wc_get_checkout_url();
                    return $urlredirect;
                }
                CpayLog::newLog("[ERROR]class-cpay.php l.".__LINE__." GET URL SMART FORM EMPTY ORDER ID ENCRYPT OR PAYMENT REQUEST ID, {asked(string) orderIdEncrypte : sent(OrderIdEncrypt:".gettype($orderIdEncrypte).") ".$orderIdEncrypte.", sent(OrderIdEncrypt:".gettype($paymentRequestId).") ".$paymentRequestId."}");
                return "";
            }
            CpayLog::newLog("[ERROR]class-cpay.php l.".__LINE__." GET URL SMART FORM BAD VALUE BREAKDOWS/ENDPOINT, {{asked(array) requete['reponse']->breakdown : sent(".gettype($requete['reponse']->breakdowns).")".json_encode($requete['reponse']->breakdowns)."},{asked(string) endpoint : sent(".gettype($requete['reponse']->breakdowns[0]->endpoint).") ".$requete['reponse']->breakdowns[0]->endpoint."}");
            return "";
        }
        CpayLog::newLog("[ERROR]class-cpay.php l.".__LINE__." GET URL SMART FORM BAD VALUE, {{asked requete : sent ".json_encode($requete)."},{asked requete['reponse'] : sent ".$requete->reponse."},{asked(int) order_id : sent(".gettype($order_id).") ".$order_id."}");
        return "";
    }

    private function load_dependencies()
    {

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cpay-log.php';
        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cpay-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cpay-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-cpay-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-cpay-public.php';

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cpay-wp-db.php';

        $this->loader = new Cpay_Loader();

    }

    private function set_locale()
    {

        $plugin_i18n = new Cpay_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');

    }

    private function define_admin_hooks()
    {

        $plugin_admin = new Cpay_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
        $this->loader->add_filter('woocommerce_payment_gateways', $plugin_admin, 'cpay_add_gateway_class');

        //$this->loader->add_action('plugins_loaded', $plugin_admin, 'cpay_init_gateway_class');

    }

    private function define_public_hooks()
    {

        $plugin_public = new Cpay_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

    }

    public function run()
    {
        $this->loader->run();
    }


    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    public function get_loader()
    {
        return $this->loader;
    }

    public function get_version()
    {
        return $this->version;
    }

}
